import iSATLibrary.ISatSolver;
import iSATLibrary.SatSolverForNative;

public class GCP {

	 int nodes = 4;
	 int colors = 3; 
	 int[][] edges = {{1,2}, {1,3}, {2,3}, {2,4}, {3,4}};

	 public int v(int i, int j){ return (i-1) * colors + j; }
	 
	 public void encode(ISatSolver solver){
		  for (int i=1; i<=nodes; i++){
			  int[] cl = new int[colors];
			  for (int j=1; j<=colors; j++)
				  cl[j-1] = v(i,j);
			  solver.addClause(cl);
		  }		  
		  for (int[] edge: edges) {
			  for (int j=1; j<=colors; j++)
				  solver.addClause(new int[] {-v(edge[0],j), -v(edge[1],j)});
		  }
	 }
	 
	 public boolean solve(ISatSolver solver){
		  return solver.solve()==10;
	 }

	 public void decode(ISatSolver solver){
		  System.out.println("SATISFIABLE");
		  int[] model = solver.getIntModel();
		  for (int i=1; i<=nodes; i++) {
				for (int j=1; j<=colors; j++)
					 if (model[v(i,j)-1] > 0) System.out.println("Node " + i + "'s color : " + j);
		  }
	 }

	 public static void main(String[] args) {
		GCP gcp = new GCP();
		ISatSolver solver = new SatSolverForNative("glueminisat");
		solver.setVerbosity(0);
		gcp.encode(solver);
		if (gcp.solve(solver)) gcp.decode(solver); else System.out.println("UNSATISFIABLE");
	 }

}
