package iSugar;

import jp.kobe_u.sugar.Logger;
import jp.kobe_u.sugar.SugarException;
import jp.kobe_u.sugar.SugarMain;
import jp.kobe_u.sugar.csp.BooleanVariable;
import jp.kobe_u.sugar.csp.CSP;
import jp.kobe_u.sugar.csp.Clause;
import jp.kobe_u.sugar.csp.IntegerVariable;
import jp.kobe_u.sugar.encoder.Encoder;
import jp.kobe_u.sugar.encoder.Problem;

/**
 * Encoder for using Sat4j.
 * @author sako
 *
 */
public class Encoder4j extends Encoder {

	public Encoder4j(CSP csp) {
		super(csp);
	}
	
	public void setVariablesCount() throws SugarException {
        problem.clear();
        int satVariablesCount = 0;
        for (IntegerVariable v : csp.getIntegerVariables()) {
            v.setCode(satVariablesCount + 1);
            int size = v.getSatVariablesSize();
            satVariablesCount += size;
        }
        for (BooleanVariable v : csp.getBooleanVariables()) {
            v.setCode(satVariablesCount + 1);
            int size = v.getSatVariablesSize();
            satVariablesCount += size;
        }
        problem.addVariables(satVariablesCount - problem.variablesCount);
	}
	
    public void encode() throws SugarException {
        int count = 0;
        int n = csp.getIntegerVariables().size();
        int percent = 10;
        for (IntegerVariable v : csp.getIntegerVariables()) {
            encodeIntegerVariable(v);
            count++;
            if ((100*count)/n >= percent) {
                Logger.fine(count + " (" + percent + "%) "
                        + "CSP integer variables are encoded"
                        + " (" + problem.summary() + ")");
                percent += 10;
            }
        }
        count = 0;
        n = csp.getClauses().size();
        percent = 10;
        for (Clause c : csp.getClauses()) {
            if (c.isValid())
                continue;
            int clausesCount0 = problem.clausesCount;
            encodeClause(c);
            count++;
            if (SugarMain.debug >= 1) {
                int k = problem.clausesCount - clausesCount0;
                Logger.fine(k + " SAT clauses for " + c);
            }
            if ((100*count)/n >= percent) {
                Logger.fine(count + " (" + percent + "%) "
                        + "CSP clauses are encoded"
                        + " (" + problem.summary() + ")");
                percent += 10;
            }
        }
        if (true) {
            int code0, code1;
            for (IntegerVariable v : csp.getIntegerVariables()) {
                if (! v.isDominant())
                    continue;
                code0 = v.getCode();
                code1 = code0 + v.getSatVariablesSize() - 1;
                if (code0 < code1)
                    problem.addPragmaDominant(code0, code1);
            }
            code0 = code1 = 0;
            for (BooleanVariable v : csp.getBooleanVariables()) {
                if (! v.isDominant())
                    continue;
                int code = v.getCode();
                if (code0 == 0) {
                    code0 = code1 = code;
                } else if (code1 + 1 == code) {
                    code1 = code;
                } else {
                    problem.addPragmaDominant(code0, code1);
                    code0 = code1 = code;
                }
            }
            if (code0 != 0) {
                problem.addPragmaDominant(code0, code1);
            }
        }
        problem.done();
        Logger.fine(count + " CSP clauses encoded");
    }

    public void encode(Problem problem) throws SugarException {
        this.problem = problem;
        if (csp.getGroups() > 0) {
            problem.setGroups(csp.getGroups(), csp.getTopWeight());
        }
        setVariablesCount();
    }

}
