package iSugar;

import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

public class Graph {
	
	private TreeSet<Integer> nodes;
	private TreeSet<int[]> edges;	
	private Map<Integer, Set<Integer>> adjacentMap;
	
	// constructor
	public Graph(){
		nodes = new TreeSet<Integer>();
		edges = new TreeSet<int[]>(new Comparator<int[]>(){
			@Override
			public int compare(int[] a0, int[] a1) {
				if (a0[0] != a1[0])
					return a0[0] < a1[0] ? -1 : 1;
				if (a0[1] != a1[1])
					return a0[1] < a1[1] ? -1 : 1;
				return 0;
			}});
		adjacentMap = new HashMap<Integer, Set<Integer>>();
	}
	
	public TreeSet<Integer> getNodes(){
		return nodes;
	}
	
	public TreeSet<int[]> getEdges(){
		return edges;
	}
	
	// generate edges
	public int[] generateEdge(int n1, int n2) {
		int[] edge = new int[2];
		if(n1 < n2){
			edge[0] = n1; edge[1] = n2;
		}
		else{
			edge[0] = n2; edge[1] = n1;
		}
		return edge;	
	}
	
	// add a condition, "n1 have n2 as a set of adjacent nodes"
	public void addAdjacent(int n1, int n2){
		if(adjacentMap.containsKey(n1))
			adjacentMap.get(n1).add(n2); // add to set of adjacent nodes
		else{
         Set<Integer> set = new LinkedHashSet<Integer>(); // generate set of adjacent nodes
			set.add(n2);
			adjacentMap.put(n1, set); // add to adjacent map
		}
	}
	
	// add a vertex to graph
	public void addNode(int n){
		nodes.add(n);
	}

	// add an edge to graph
	public void addEdge(int n1, int n2){
		if(n1 != n2){
			int[] edge = generateEdge(n1,n2);
				edges.add(edge);
				addAdjacent(n1, n2);
				addAdjacent(n2, n1);				
		}	
	}	

	// get the set of adjacent nodes
	public Set<Integer> getAdjacent(int n){
		return adjacentMap.get(n);
	}
	
}
