package iSugar;

import static iSugar.Solver.Status.*;

import java.io.IOException;

import jp.kobe_u.sugar.SugarException;

/**
 * This class is for solving MaxCSP.
 * As back-end solver only glueminisat is available.
 * @author sako
 *
 */
public class MaxCSPSolver extends Solver {

	public MaxCSPSolver(String solverName) {
		super(solverName);
	}

	/**
	 * Solves MaxCSP.
	 * @throws SugarException
	 * 
	 * @throws IOException 
	 * 
	 */
	public void solveProblem() throws SugarException, IOException {

		boolean isSAT = false;
		int numberOfSolve = 0;
		double solveTime, optSolveTime;
		int optCost = 0;
		int lb;
		int ub;

		//MaxCSP.isWeighted = true;
		encode();
		maxcsp.commit();
		maxcsp.registerRelaxationVariables();
		lb = -1; ub = 1 + maxcsp.getSumOfWeight();
		
		optSolveTime = getTime();
		while(lb+1 < ub){
			int midval = (lb+ub)/2;
			int[] assumptions = maxcsp.getAssumptions();
			addAssumptions(assumptions);
			maxcsp.addTempSumOfWeightsConstraint(this, midval);
			System.out.println("c " + ++numberOfSolve + "th Solving.");
			System.out.println("c OBJECTIVE BOUND: " + lb + " <= _COST <= " + ub);
			int bound = Math.min(maxcsp.getBound(),midval);
			System.out.println("c SEARCHING: _COST <= " + bound);
			solveTime = getTime();
			int res = solveCSP();
			if(res == SATISFIABLE){
				solveTime = getTime() - solveTime;
				isSAT = true;
				System.out.println("s SATISFIABLE");
				optCost = maxcsp.getCost();
				System.out.printf("c Solving time : %f sec\n", solveTime);
				System.out.println("c OBJECTIVE _COST " + optCost);
				System.out.println("o " + optCost);
				outputSolution();
				System.out.print("c REFINE UPPERBOUND : " + ub + " => ");
				ub = maxcsp.refineUpperBound(ub);
				System.out.println(ub);
			} else {
				solveTime = getTime() - solveTime;
				System.out.println("s UNSATISFIABLE");
				System.out.println("c NOT FOUND: _COST <= " + bound);
				System.out.printf("c Solving time : %f sec\n", solveTime);
				int[] resAssumptions = getFailed(assumptions);
				if (!maxcsp.updateAssumptionLiterals(resAssumptions)){
					System.out.print("c REFINE LOWERBOUND : " + lb + " => ");
					lb = maxcsp.refineLowerBound(midval)-1;
					System.out.println(lb);
				}
			}
		}
		if(isSAT){
			System.out.println("c OBJECTIVE _COST " + optCost);
			System.out.println("s OPTIMUM FOUND");
		} else {
			System.out.println("c SOLUTION NOT FOUND");			
		}
		optSolveTime = getTime() - optSolveTime;
		System.out.printf("c Number of Solves : %d\n", numberOfSolve);
		System.out.printf("c Total Solving Time : %f sec\n", optSolveTime);	
	}
	
}
